/*
 * idr.h: Vercel UD-1 (IDR) specific defines
 *
 * Copyright (C) 2001 Tim Riker <Tim@Rikers.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: idr.h,v 1.2 2001/12/16 07:38:50 timriker Exp $"

#ifndef BLOB_ARCH_IDR_H
#define BLOB_ARCH_IDR_H

/* boot CPU speed */
#define CPU_SPEED		(0x05)

/* serial port */
#define USE_SERIAL3

/* GPIO for the LED */
#define LED_GPIO		(0x00000000) /* safe mode: no GPIO, no LED */

/* the base address were BLOB is loaded by the first stage loader */
#define BLOB_ABS_BASE_ADDR	(0xc0200400)

/* where do various parts live in RAM */
#define BLOB_RAM_BASE		(0xc0100000)
#define KERNEL_RAM_BASE		(0xc0008000)
#define PARAM_RAM_BASE		(0xc0110000)
#define RAMDISK_RAM_BASE	(0xc0400000)

/* and where do they live in flash */
#define BLOB_FLASH_BASE		(0x00000000)
#define BLOB_FLASH_LEN		(256 * 1024)
#define PARAM_FLASH_BASE	(BLOB_FLASH_BASE + BLOB_FLASH_LEN)
#define PARAM_FLASH_LEN		(256 * 1024)
#define KERNEL_FLASH_BASE	(PARAM_FLASH_BASE + PARAM_FLASH_LEN)
#define KERNEL_FLASH_LEN	(1024 * 1024)
#define RAMDISK_FLASH_BASE	(KERNEL_FLASH_BASE + KERNEL_FLASH_LEN)
#define RAMDISK_FLASH_LEN	(4 * 1024 * 1024)

/* the position of the kernel boot parameters */
#define BOOT_PARAMS		(0xc0000100)

/* the size (in kbytes) to which the compressed ramdisk expands */
#define RAMDISK_SIZE		(8 * 1024)

/* Memory configuration */
#ifdef BLOB_NEED_MEMCONFIG
#warning "use defines from memsetup.h for better readability"
#	define MDCNFG_VALUE	0x72347235		/* 0x0  MDCNFG */
#	define MDCAS00_VALUE	0x55555557		/* 0x04 MDCAS00 */
#	define MDCAS01_VALUE	0x55555555		/* 0x08 MDCAS01 */
#	define MDCAS02_VALUE	0x55555555		/* 0x0c MDCAS02 */
#	define MSC0_VALUE	0x00004B94		/* 0x10 MCS0 */
#	define MSC1_VALUE	0x22212419		/* 0x14 MCS1 */
#	define MECR_VALUE	0x00000000		/* 0x18 MECR */
#	define MDREFR_VALUE	DO_NOT_USE_THIS_VALUE__GETS_AUTOMAGICALLY_COMPUTED
#	define MDCAS20_VALUE	0x55555557		/* 0x20 MDCAS20 */
#	define MDCAS21_VALUE	0x55555555		/* 0x24 MDCAS21 */
#	define MDCAS22_VALUE	0x55555555		/* 0x28 MDCAS22 */
#	define MSC2_VALUE	0x44396669		/* 0x2C MCS2 */
#	define SMCNFG_VALUE	0xA040A040		/* 0x30 SMCNFG */
#endif

#endif
