/*
 * lart.c: LART specific stuff
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ident "$Id: lart.c,v 1.6 2002/01/03 16:07:18 erikm Exp $"

#ifdef HAVE_CONFIG_H
# include <blob/config.h>
#endif

#include <blob/flash.h>
#include <blob/init.h>
#include <blob/serial.h>




/* flash descriptor for LART flash */
/* 2x Intel 28F160F3B fast boot block flash (4MB) */
static flash_descriptor_t lart_flash_descriptors[] =
{
	{ 
		size: 2 * 8 * 1024,
		num: 8,
	},
	{
		size: 2 * 64 * 1024,
		num: 31,
	},
	{
		/* NULL block */
	},
};




static void init_lart_flash_driver(void)
{
	/* we could do funky detection over here, because the LART can
	 * have both internal and external flash with different
	 * properties. for the time being we just ignore that fact.
	 *   -- Erik
	 */

	flash_descriptors = lart_flash_descriptors;
	flash_driver = &intel32_flash_driver;
}

__initlist(init_lart_flash_driver, INIT_LEVEL_DRIVER_SELECTION);




static void lart_init_hardware(void)
{
	/* select serial driver */
	serial_driver = &sa11x0_serial_driver;
}

__initlist(lart_init_hardware, INIT_LEVEL_DRIVER_SELECTION);
